#include <array>
#include <iostream>


/**
 * @brief Array-to-pointer decay example, 
 *       when a C-style array is passed to a function by value, array decays into pointer
 * @param arr  array passed in as a value
 */
void print_arr(int arr[]){ 
     size_t arr_size = sizeof(arr) / sizeof(int); // This will not result to the number of element in the array
     std::cout << "In array-to-pointer decay example, \nsizeof(arr)/sizeof(int) = " << arr_size << std::endl;
     for(int i = 0; i < arr_size; ++i){
          std::cout << arr[i] << std::endl;
     }
}

/**
 * @brief Solution1 : pass the array size as paramter
 * @param arr  array passed in as a value, which decays to a pointer
 * @param arr_size the number of element in of arr
 */
void print_arr(int arr[], size_t arr_size){ 
     std::cout << "solution1 Array size is: " << arr_size << std::endl;
     for(int i = 0; i < arr_size; ++i){
          std::cout << arr[i] << std::endl;
     }
}

/**
 * @brief Solution2 : use defined size std::array
 *        More solutions, use std::vector, or template this function with the array size, 
 *        we will learn more in the future recitations about template
 * @param arr  std::array<T,N>
 * @param arr_size the number of element in of arr
 */
void print_arr(std::array<int, 5> arr){ 
     std::cout << "soltion2 Array size is: " << arr.size() << std::endl;
     for(int i = 0; i < arr.size(); ++i){
          std::cout << arr[i] << std::endl;
     }
}



int main(){
     // std::array<T, N>
     std::array<int, 5> my_arr = {1, 2, 3, 4, 5};
     // accessing element in std::array 
     std::cout << "my_arr element at index=3 is: " << my_arr.at(3) << std::endl;
     // get the total number of elements in std::array 
     std::cout << "my_arr size is: " << my_arr.size()<< std::endl;
     // use std::array iterator
     for(auto it = my_arr.begin(); it < my_arr.end(); ++it){
          std::cout << *it << std::endl;
     }
     

     // C-style array 
     int raw_arr[10] = {0}; 
     /**
      * @brief Need bound check when accessing element using operator[]
      *        If indexing element with pos > 10, like raw_arr[1000], the value is undefined. 
      *        out of bound indexing should be avoided in programs.
      */
     std::cout << "raw_arr element at index=1 is: " << raw_arr[1] << std::endl; 
     /**
      * @brief number of elements in the array = total memory occupied by the array / the size of each element
      *        The sizeof() operator on an array returns the total memory occupied by the array in bytes.      
      *        This function works as long as it is in the same scope as raw_arr[]
      */
     std::cout << "raw_arr size is: " << sizeof(raw_arr) / sizeof(int) << std::endl;
     // example of array-to-pointer decay
     print_arr(raw_arr);
     
     // call solution1 function
     print_arr(raw_arr, 10);
     // call solution2 function
     print_arr(my_arr);

     return 0;
}